'use strict';

/**
 * 错误基类
 * @class BaseError
 * @author: yeliex
 * @create: 2017/8/23 下午3:58
 */
const LevelType = require('./LevelType');
const DomainTypes = require('./DomainTypes');
const SeriesTypes = require('./SeriesTypes');
const SourceTypes = require('./SourceTypes');
const PriorityTypes = require('./PriorityTypes');
const { UNKNOWN_CODE } = require('./ErrorCodes');

var _$domainType = new WeakMap();

var _$seriesType = new WeakMap();

var _$source = new WeakMap();

var _$priority = new WeakMap();

var _$levelType = new WeakMap();

var _$code = new WeakMap();

var _$message = new WeakMap();

var _$cause = new WeakMap();

var _$readOnly = new WeakMap();

class BaseError {
  constructor() {
    _$domainType.set(this, DomainTypes.B);

    _$seriesType.set(this, SeriesTypes.COMMON);

    _$source.set(this, SourceTypes.INNER);

    _$priority.set(this, PriorityTypes.P0);

    _$levelType.set(this, LevelType.ERROR);

    _$code.set(this, UNKNOWN_CODE);

    _$message.set(this, '');

    _$readOnly.set(this, {});
  }

  /**
   * 将自己clone一份导出
   * @returns {Error}
   */
  copy() {
    const errors = new BaseError();

    errors.domainType = this.domainType;
    errors.code = this.code;
    errors.levelType = this.levelType;
    errors.message = this.message;
    errors.cause = this.cause;
    errors.priority = this.priority;
    errors.seriesType = this.seriesType;
    errors.source = this.source;

    return errors;
  }

  get writableErrors() {
    return this.copy();
  }

  isWarning() {
    return _$levelType.get(this) ? _$levelType.get(this).level === LevelType.WARNING.level : false;
  }

  isError() {
    return _$levelType.get(this) ? _$levelType.get(this).level === LevelType.ERROR.level : false;
  }

  get domainType() {
    return _$domainType.get(this);
  }

  set domainType(domainType) {
    _$domainType.set(this, domainType);
  }

  setDomainType(domainType) {
    this.domainType = domainType;
    return this;
  }

  get seriesType() {
    return _$seriesType.get(this);
  }

  set seriesType(seriesType) {
    _$seriesType.set(this, seriesType);
  }

  setSeriesType(seriesType) {
    this.seriesType = seriesType;
    return this;
  }

  get source() {
    return _$source.get(this);
  }

  set source(source) {
    _$source.set(this, source);
  }

  setSource(source) {
    this.source = source;
    return this;
  }

  get priority() {
    return _$priority.get(this);
  }

  set priority(priority) {
    _$priority.set(this, priority);
  }

  setPriority(priority) {
    this.priority = priority;
    return this;
  }

  get code() {
    return _$code.get(this);
  }

  set code(code) {
    _$code.set(this, code);
  }

  setCode(code) {
    this.code = code;
    return this;
  }

  get cause() {
    return _$cause.get(this);
  }

  set cause(cause) {
    _$cause.set(this, cause);
  }

  setCause(cause) {
    this.cause = cause;
    return this;
  }

  get levelType() {
    return _$levelType.get(this);
  }

  set levelType(levelType) {
    _$levelType.set(this, levelType);
  }

  setLevelType(levelType) {
    this.levelType = levelType;
    return this;
  }

  get message() {
    return _$message.get(this) ? _$message.get(this).toString() : _$code.get(this).comment;
  }

  set message(message) {
    _$message.set(this, String(message));
  }

  setMessage(message) {
    this.message = message;
    return this;
  }

  appendMsg(str) {
    _$message.set(this, str);
  }

  get codeString() {
    return `${_$domainType.get(this).toString()}${[_$levelType.get(this).toString(), this.seriesType, `${this.source}${this.priority}`, this.code].join('-')}`;
  }

  get codeStringWithMsg() {
    return this.message ? `${this.codeString} ${this.message}` : this.codeString;
  }

  setRetryStratgy(times, interval) {
    throw new Error('not supported', this);
  }

  get hashCode() {
    let result = this.domainType ? this.domainType.hashCode : 0;
    result = 31 * result + (this.seriesType ? this.seriesType.hashCode : 0);
    result = 31 * result + (this.source ? this.source.hashCode : 0);
    result = 31 * result + (this.priority ? this.priority.hashCode : 0);
    result = 31 * result + (this.levelType ? this.levelType.hashCode : 0);
    result = 31 * result + (this.code ? this.code.hashCode : 0);
    result = 31 * result + (this.message ? this.message.hashCode : 0);
    result = 31 * result + (this.cause ? this.cause.hashCode : 0);
    result = 31 * result;
    result = 31 * result + (this.readOnly ? 1 : 0);

    return result;
  }

  get readOnly() {
    return _$readOnly.get(this);
  }

  set readOnly(readOnly) {
    _$readOnly.set(this, readOnly);
  }

  setReadOnly(readOnly) {
    this.readOnly = readOnly;
    return this;
  }

  toString() {
    return this.codeStringWithMsg;
  }

  equals(error) {
    if (!error instanceof BaseError) {
      return false;
    }

    return !['readOnly', 'domainType', 'equals', 'source', 'priority', 'levelType', 'code', 'message', 'cause', 'retry'].some(key => {
      const current = this[key];
      const next = error[current];
      return typeof current === 'object' ? !current.equal(next) : current !== next;
    });
  }
}

module.exports = BaseError;
//# sourceMappingURL=BaseError.js.map